<?php
/**
 * @copyright: Copyright © 2020 Firebear Studio. All rights reserved.
 * @author   : Firebear Studio <fbeardev@gmail.com>
 */
namespace Firebear\PlatformSap\Cron;

use Magento\Framework\Stdlib\DateTime;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Firebear\PlatformSap\Model\ResourceModel\Log;

class LogCleanup
{
    /**
     * @var DateTime
     */
    private $dateTime;

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * @var \Magento\Framework\Stdlib\DateTime\DateTime
     */
    private $date;

    /**
     * @var Log
     */
    private $resource;

    /**
     * BulkCleanup constructor.
     * @param DateTime $dateTime
     * @param ScopeConfigInterface $scopeConfig
     * @param DateTime\DateTime $time
     */
    public function __construct(
        DateTime $dateTime,
        ScopeConfigInterface $scopeConfig,
        \Magento\Framework\Stdlib\DateTime\DateTime $time,
        Log $resource
    ) {
        $this->dateTime = $dateTime;
        $this->scopeConfig = $scopeConfig;
        $this->date = $time;
        $this->resource = $resource;
    }

    /**
     * Remove all expired sap integration logs
     *
     * @return void
     */
    public function execute()
    {
        $isEnableClearLog = $this->scopeConfig->getValue('firebear_importexport/sap/sap_clear_log');

        if ($isEnableClearLog) {
            $connection = $this->resource->getConnection();
            $logLifetime = 3600 * 24 * (int)$this->scopeConfig->getValue('firebear_importexport/sap/sap_log_lifetime');
            $maxLogUpdateAtTime = $this->dateTime->formatDate($this->date->gmtTimestamp() - $logLifetime);
            $connection->delete($this->resource->getMainTable(), ['updated_at <= ?' => $maxLogUpdateAtTime]);
        }
    }
}
