<?php
 /**
  * Copyright © Firebear Studio, Inc. All rights reserved.
  * See COPYING.txt for license details.
  */
namespace Firebear\PlatformSap\Model\Import\Source;

/**
 * Sap category source
 */
class Category extends AbstractSource
{
    const DEFAULT_CATEGORY_NAME = 'Default Category';

    /**
     * @var \Firebear\PlatformSap\Model\Source\Gateway\Category
     */
    protected $gateway;

    /**
     * Config data
     *
     * @var array
     */
    protected $_data = [];

    /**
     * @var \Magento\Framework\App\RequestInterface
     */
    protected $request;

    /**
     * Product constructor.
     * @param Gateway\Product $gateway
     * @param \Magento\Framework\App\RequestInterface $request
     * @param null $data
     * @throws \Exception
     */
    public function __construct(
        \Firebear\PlatformSap\Model\Import\Source\Gateway\Category $gateway,
        \Magento\Framework\App\RequestInterface $request,
        $data = null
    ) {
        $this->gateway = $gateway;
        $this->request = $request;
        $this->parseEntities($data);
    }

    /**
     * @param $data
     * @return array|bool
     * @throws \Exception
     */
    protected function parseEntities($data)
    {
        $this->entities = parent::parseEntities($data);
        if (!empty($this->entities)) {
            foreach ($this->entities as $key => $entity) {
                if (isset($entity['ItmsGrpNam'])) {
                    $this->entities[$key]['name'] = self::DEFAULT_CATEGORY_NAME .
                        '/' . $entity['ItmsGrpNam'];
                    $this->entities[$key]['available_sort_by'] = 'name';
                    $this->entities[$key]['include_in_menu'] = 1;
                    $this->entities[$key]['is_active'] = 1;
                    $this->entities[$key]['default_sort_by'] = 'Position';
                }
            }
        }
        $this->_colNames = array_keys($this->entities[0] ?? []);
        return $this->entities;
    }
}
