<?php
/**
 * Copyright © Firebear Studio GmbH, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace Firebear\PlatformSap\Model;

use Firebear\PlatformSap\Api\InvoiceOrderInterface;
use Magento\Sales\Api\Data\InvoiceCommentCreationInterface as InvoiceCommentCreationInterface;
use Magento\Sales\Api\Data\InvoiceCreationArgumentsInterface;
use Magento\Sales\Model\Order\OrderStateResolverInterface;
use Magento\Sales\Model\OrderFactory;
use Firebear\PlatformSap\Model\Export\Adapter\Order;
use Magento\Sales\Api\Data\OrderInterface;

/**
 * InvoiceOrder model
 */
class InvoiceOrder implements InvoiceOrderInterface
{
    /**
     * @var \Magento\Sales\Api\InvoiceOrderInterface
     */
    protected $invoiceOrder;

    /**
     * @var OrderFactory
     */
    protected $orderFactory;

    /**
     * InvoiceOrder constructor.
     * @param \Magento\Sales\Api\InvoiceOrderInterface $invoiceOrder
     * @param OrderFactory $orderFactory
     */
    public function __construct(
        \Magento\Sales\Api\InvoiceOrderInterface $invoiceOrder,
        OrderFactory $orderFactory
    ) {
        $this->invoiceOrder = $invoiceOrder;
        $this->orderFactory = $orderFactory;
    }

    /**
     * @param int $orderId
     * @param bool $capture
     * @param \Firebear\PlatformSap\Api\Data\ShipmentItemCreationInterface[] $items
     * @param bool $notify
     * @param bool $appendComment
     * @param \Magento\Sales\Api\Data\InvoiceCommentCreationInterface|null $comment
     * @param \Magento\Sales\Api\Data\InvoiceCreationArgumentsInterface|null $arguments
     * @return int
     * @throws \Magento\Sales\Api\Exception\DocumentValidationExceptionInterface
     * @throws \Magento\Sales\Api\Exception\CouldNotInvoiceExceptionInterface
     * @throws \Magento\Framework\Exception\InputException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     * @throws \DomainException
     */
    public function execute(
        $orderId,
        $capture = false,
        array $items = [],
        $notify = false,
        $appendComment = false,
        InvoiceCommentCreationInterface $comment = null,
        InvoiceCreationArgumentsInterface $arguments = null
    ) {

        $order = $this->orderFactory->create()->loadByAttribute(OrderInterface::INCREMENT_ID, $orderId);
        $orderId = $order->getId() ? $order->getId() : $orderId;

        foreach ($items as $item) {
            foreach ($order->getItems() as $orderItem) {
                if ($item->getSku() == $orderItem->getSku()) {
                    $item->setOrderItemId($orderItem->getItemId());
                }
            }
        }

        return $this->invoiceOrder->execute(
            $orderId,
            $capture,
            $items,
            $notify,
            $appendComment,
            $comment,
            $arguments
        );
    }
}

