<?php
/**
 * Copyright © Firebear Studio GmbH, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace Firebear\PlatformSap\Model;

use Firebear\PlatformSap\Api\ShipOrderInterface;
use Magento\Sales\Api\Data\ShipmentCommentCreationInterface as ShipmentCommentCreationInterfaceAlias;
use Magento\Sales\Model\OrderFactory;
use Firebear\PlatformSap\Model\Export\Adapter\Order;

/**
 * Class ShipOrder
 */
class ShipOrder implements ShipOrderInterface
{
    protected $shipOrder;

    /**
     * @var OrderFactory
     */
    protected $orderFactory;

    public function __construct(
        \Magento\Sales\Api\ShipOrderInterface $shipOrder,
        OrderFactory $orderFactory
    ) {
        $this->shipOrder = $shipOrder;
        $this->orderFactory = $orderFactory;
    }

    /**
     * @param string                                                             $orderId
     * @param \Firebear\PlatformSap\Api\Data\ShipmentItemCreationInterface[]  $items
     * @param bool                                                            $notify
     * @param bool                                                            $appendComment
     * @param ShipmentCommentCreationInterfaceAlias|null                      $comment
     * @param \Magento\Sales\Api\Data\ShipmentTrackCreationInterface[]        $tracks
     * @param \Magento\Sales\Api\Data\ShipmentPackageCreationInterface[]      $packages
     * @param \Magento\Sales\Api\Data\ShipmentCreationArgumentsInterface|null $arguments
     *
     * @return int
     * @throws \Magento\Sales\Api\Exception\DocumentValidationExceptionInterface
     * @throws \Magento\Sales\Api\Exception\CouldNotShipExceptionInterface
     * @throws \Magento\Framework\Exception\InputException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     * @throws \DomainException
     */
    public function execute(
        $orderId,
        array $items = [],
        $notify = false,
        $appendComment = false,
        ShipmentCommentCreationInterfaceAlias $comment = null,
        array $tracks = [],
        array $packages = [],
        \Magento\Sales\Api\Data\ShipmentCreationArgumentsInterface $arguments = null
    ) {
        $order = $this->orderFactory->create()->loadByAttribute(\Magento\Sales\Api\Data\OrderInterface::INCREMENT_ID, $orderId);
        $orderId = $order->getId() ? $order->getId() : $orderId;

        foreach ($items as $item) {
            foreach ($order->getItems() as $orderItem) {
                if ($item->getSku() == $orderItem->getSku()) {
                    $item->setOrderItemId($orderItem->getItemId());
                }
            }
        }

        return $this->shipOrder->execute(
            $orderId,
            $items,
            $notify,
            $appendComment,
            $comment,
            $tracks,
            $packages,
            $arguments
        );
    }
}
